/*
 * Copyright 2005 JBoss Inc
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.drools.core.event;

import org.drools.core.FactHandle;
import org.drools.core.WorkingMemory;
import org.drools.core.spi.PropagationContext;

public class ObjectRetractedEvent extends WorkingMemoryEvent {
    private static final long serialVersionUID = 510l;

    private final FactHandle  handle;

    private final Object      oldObject;
    
    private final int         type;

    public ObjectRetractedEvent(final WorkingMemory workingMemory,
                                final PropagationContext propagationContext,
                                final FactHandle handle,
                                final Object oldObject) {
        super( workingMemory,
               propagationContext );
        this.handle = handle;
        this.oldObject = oldObject;
        this.type = propagationContext.getType();
    }

    public FactHandle getFactHandle() {
        return this.handle;
    }

    public Object getOldObject() {
        return this.oldObject;
    }

    public int getType() {
        return this.type;
    }
    
    public String toString() {
        return "[ObjectRetracted: type="+PropagationContext.typeDescr[type]+" handle=" + this.handle + "; old_object=" + this.oldObject + "]";
    }
    
}
